#include <json.hpp>
#include "ProfinetSniffer.hpp"

using namespace cybus_profinet;
namespace vrpc {

  // Register DeviceInfo
  inline void to_json(json& j, const DeviceInfo& di) {
    j = json{
      {"deviceName", di.device_name},
      {"vendorId", di.vendor_id},
      {"ip", di.ip},
      {"mac", di.mac},
      {"deviceId", di.device_id},
      {"deviceType", di.device_type}
    };
  }

  inline void from_json(const json& j, DeviceInfo& di) {
    di.device_name = j.at("deviceName").get<std::string>();
    di.vendor_id = j.at("vendorId").get<std::uint16_t>();
    di.device_id = j.at("vdeviceId").get<std::uint16_t>();
    di.device_type = j.at("deviceType").get<std::string>();
    di.ip = j.at("ip").get<std::string>();
    di.mac = j.at("mac").get<std::string>();
  }

  // Register ProfinetValue
  inline void to_json(json& j, const ProfinetValue& pv) {
    j = json{
      {"isError", pv.is_error},
      {"buffer", pv.buffer},
      {"dataType", pv.data_type},
      {"slot", pv.slot},
      {"subslot", pv.subslot},
      {"propertyIndex", pv.property_index}
    };
  }

  inline void from_json(const json& j, ProfinetValue& pv) {
    //pv.is_error = j.at("isError").get<bool>();
    //pv.buffer = j.at("buffer").get<std::vector<char> >();
    //pv.data_type = j.at("dataType").get<int32_t>();
  }

  // Register public interface
  VRPC_CTOR(ProfinetSniffer,
    const std::string&,
    const std::string&,
    const std::string&
  )

  VRPC_VOID_MEMBER_FUNCTION(ProfinetSniffer,
    start
  )

  VRPC_VOID_MEMBER_FUNCTION(ProfinetSniffer,
    stop
  )

  VRPC_MEMBER_FUNCTION(ProfinetSniffer,
    std::vector<DeviceInfo>,
    getDevices
  )

  VRPC_VOID_MEMBER_FUNCTION(ProfinetSniffer,
    reparseGsdFiles
  )

  VRPC_VOID_MEMBER_FUNCTION(ProfinetSniffer,
    subscribe,
    const std::string&, // device_name
    int32_t, // api
    int32_t, // slot
    int32_t, // sub_slot
    int32_t, // property_index
    VRPC_CALLBACK(ProfinetValue)
  )

  VRPC_VOID_MEMBER_FUNCTION(ProfinetSniffer,
    subscribe,
    const std::string&, // device_name
    int32_t, // api
    int32_t, // slot
    int32_t, // sub_slot
    const std::string&, // property_name
    VRPC_CALLBACK(ProfinetValue)
  )

  VRPC_VOID_MEMBER_FUNCTION(ProfinetSniffer,
    unsubscribe,
    const std::string&, // device_name
    int32_t, // api
    int32_t, // slot
    int32_t, // sub_slot
    int32_t // property_index
  )

  VRPC_VOID_MEMBER_FUNCTION(ProfinetSniffer,
    unsubscribe,
    const std::string&, // device_name
    int32_t, // api
    int32_t, // slot
    int32_t, // sub_slot
    const std::string& // property_name
  )
}
